/***************************************************************************
 *   Copyright (C) 2002 by Rodolfo Conde                                   *
 *   aragorn@ciencias.unam.mx                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

import java.util.LinkedList;
import java.util.Iterator;

 
/** A command, which one is, and all the arguments of it. */
public class Command extends IdentifiedWithPosition {
	/* The commands */
	
	/** Move command */
	public static final int MOVE = 0;
	
	/** Pick command */
	public static final int PICK = 1;
	
	/** Drop command */
	public static final int DROP = 2;
	
	public static final String commands[] = { "MOVE", "PICK", "DROP" };
	
	/** A place to store packages */
	private PackageBox pb;
	
	/** Cost of executing this command */
	int bid;
	
	/* make a new command
	 * @param comm the command
	 * @param x x coordinate for Move command
	 * @param y y coordinate for Move command
	 * @param b bidding of this command
	 *
	 */
	public Command(int comm, int x, int y, int b)
	{
		super(comm, x, y);
		bid = b;
		pb = new PackageBox();
	}
	
	/** Default Constructor */
	public Command()
	{
		this(MOVE, 0, 0, 1);
	}
	
	/**
	 * Add a package for the PICK or DROP commands
	 * @param p package to be added
	 *
	 */
	void addPackage(Package p)
	{
		pb.add(p);
	}
	
	/** sets the bid of the command
	 * @param b the bid
	 *
	 */
	 public void setBid(int b)
	 {
	 	bid = b;
	 }

	public LinkedList packs()
	{
		return pb;
	}
	 
	 /** Returns the bid of the command
	  *
		* @return the bid of the command
		*
		*/
		public int bid()
		{
			return bid;
		}
		
		/** returns an iterator of packages arguments
		 *
		 * @return an iterator of packages
		 *
		 */
		 Iterator arguments()
		 {
		 	return pb.iterator();
		 }
		 
	/** a little wrapper class */
	private class PackageBox extends LinkedList {
		public PackageBox() {
			super();
		}
		
		/** this is so we dont have to see everywhere casts
		 * @param obj object to add to the container
		 *
		 */
// 		 public void add(Object obj)
// 		 {
// 		 	
// 		 }
	}
}
