/***************************************************************************
 *   Copyright (C) 2002 by Rodolfo Conde                                   *
 *   aragorn@ciencias.unam.mx                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/


import java.lang.System;
import java.lang.Integer;
import java.util.StringTokenizer;
import java.util.LinkedList;
import java.util.Iterator;
import java.lang.NumberFormatException;
import java.net.Socket;
import java.io.BufferedReader;
import java.io.PrintWriter;
import java.io.IOException;


/**
 * This class is the player, it has the robot and tries to
 * follow whats happening....
 *
 */
public class Player {
	/** Directions */

	public static final int NORTH = 0;
	public static final int SOUTH = 1;
	public static final int WEST = 2;
	public static final int EAST = 3;

	public static final String dirs[] = { "N", "S", "W", "E" };

	/** The robot */
	private Robot robot;

	/** The map */
	private GameMap map;

	/** The Communications manager */
	private Communicator comm;

	/** The package the robot is carrying */
	LinkedList packs;

	/** Constructs a new Player */
	public Player()
	{
		robot  = null;
		map = null;
		robot = null;
		packs = new LinkedList();
	}

	/** The function where it all starts
	 * @param sname The name of the server
	 * @param sport The port where the host is listening
	 *
	 */
	public void play(String sname, int sport)
	{
		comm = new Communicator(sname, sport);

		comm.requestJoinGame();				
		map = comm.getBoard();
		robot = comm.getConfiguration();
		
		/* now we process the initial response */
		StringTokenizer stok = comm.getServerResponse();
		int x, y;

		x = y = -1;
		while (stok.hasMoreTokens()) {
			char[] str= stok.nextToken().toCharArray();
			str[0] = '0';
System.out.println(str);
			int isMyId = Integer.parseInt(new String(str));

			if (isMyId == robot.id()) {
				stok.nextToken();  // skip X
				x = Integer.parseInt(stok.nextToken());
				stok.nextToken();  // skip Y
				y = Integer.parseInt(stok.nextToken());
				System.out.println("x: " + x + " " + "y: " + y);
				break;
			}
		}
		
		robot.setPos(x, y);

		/* now enter the game loop */
		gameMainLoop();
	}

	private void gameMainLoop()
	{
		try {
			Command cmd;
			int to_where = -1, x_pos, y_pos;
			String dir;

			while (comm.connected()) {
				/* get the packages */
				StringTokenizer pstok = comm.getServerResponse();
				Package p;
				x_pos = robot.x();
				y_pos = robot.y();
				dir = null;
				cmd = new Command();
				cmd.setId(-1);

				if (packs.size() > 0) {
					Iterator it = packs.iterator();

					do {
						p = (Package) it.next();
						int x = p.x(), y = p.y();

						if (x_pos == x && y_pos == y) {
							cmd.setId(Command.DROP);
							cmd.addPackage(p);
							it.remove();
						}
					} while (it.hasNext());
					
					comm.sendCommand(cmd, dir);
					robot.execute(cmd);
					return;
				}	
				
				if (!pstok.hasMoreTokens()) {
					cmd.setId(Command.MOVE);
					cmd.setPos(x_pos, y_pos);

					if (x_pos + 1 <= map.width() && map.getAt(x_pos + 1, y_pos) != GameMap.WATER) {
						to_where = EAST;
						cmd.setX(x_pos + 1);
					}
					else if (y_pos - 1 >= 1 && map.getAt(x_pos, y_pos - 1) != GameMap.WATER) {
						to_where = SOUTH;
						cmd.setY(y_pos - 1);
					}
					else if (x_pos - 1 >= 1 && map.getAt(x_pos - 1, y_pos) != GameMap.WATER) {
						to_where = WEST;
						cmd.setX(x_pos - 1);
					}
					else if (y_pos + 1 <= map.height() && map.getAt(x_pos - 1, y_pos) != GameMap.WATER) {
						to_where = NORTH;
						cmd.setY(y_pos + 1);
					}

					dir = dirs[to_where];
				} else {
					cmd.setId(Command.PICK);

					do {
						p = new Package(Integer.parseInt(pstok.nextToken()),
								Integer.parseInt(pstok.nextToken()),
								Integer.parseInt(pstok.nextToken()),
								Integer.parseInt(pstok.nextToken()));

						cmd.addPackage(p);
					} while (pstok.hasMoreTokens());
				}
			
				comm.sendCommand(cmd, dir);
				robot.execute(cmd);
				packs.addAll(cmd.packs());
			}
		} catch (Exception ex) {}
			
	}

	/** This is the class that is the bridge
	 * between the player and the server
	 *
	 */
	private class Communicator {
		/** The communication's channel */
		private Socket s;

		/** The output stream associated with the socket */
		private PrintWriter out;

		/** The input stream associated with the socket */
		private BufferedReader in;

		/** Constructs a communicator
		 * @param sname hostname of the server
		 * @param sport port of the server
		 *
		 */
		public Communicator(String sname, int sport)
		{
			try {
				Socket s = new Socket(java.net.InetAddress.getByName(sname), sport);
	
				out = new PrintWriter(s.getOutputStream(), true);
				in = new BufferedReader(new java.io.InputStreamReader(s.getInputStream()));
			} catch (java.net.UnknownHostException uhex) {
				System.out.println("Unknow host: " + sname);
				System.exit(1);
			} catch (java.io.IOException ioex) {
				System.out.println("IO Error !!");
				System.exit(1);
			}
		}

		/** Request joinning the game */
		public void requestJoinGame()
		{
			out.println("Player");
		}

		/** Get the board from the server */
		public GameMap getBoard()
		{
			int h, w, i;
			char the_map[][];

			/** we go for the board */
			StringTokenizer stok = null;

			h = w = 0;
			the_map = null;

			try {
				/* get the dimensions */
				stok = new StringTokenizer(in.readLine());

				w = Integer.parseInt(stok.nextToken());
				h = Integer.parseInt(stok.nextToken());
				
				the_map = new char[h][];
				for (i = 0; i < h; ++i) {
					String line = in.readLine();

					System.out.println(line);
					the_map[i] = line.toCharArray();
				}
			} catch (IOException ioex) { System.out.println("Exception !!"); }

			return new GameMap(w, h, the_map);
				
		}

		/** Get the robot's configuration */
		public Robot getConfiguration()
		{
			int id, ccap, money;

			id = ccap = money = -1;

			try {
				StringTokenizer stok = new StringTokenizer(in.readLine());

				id = Integer.parseInt(stok.nextToken());
				ccap = Integer.parseInt(stok.nextToken());
				money = Integer.parseInt(stok.nextToken());
			} catch (IOException ioex) {}

			return new Robot(id, ccap, money);
		}		

		/** get standar responses from server.. */
		StringTokenizer getServerResponse()
		{
			try {
				return new StringTokenizer(in.readLine());
			} catch (IOException ioex) {}

			return null;
		}

		/** check is connected */
		public boolean connected()
		{
			return s.isConnected();
		}

		/** send the command to the server */
		void sendCommand(Command cmd, String dir)
		{
			int comm = cmd.id();
			String strcomm = null;

			strcomm = Command.commands[comm];

			if (comm == Command.MOVE)
				strcomm += " " + dir;
			else {
				Iterator it = cmd.arguments();

				while (it.hasNext()) {
					strcomm += " " + Integer.toString(((Package) it.next()).id());
				}
			}

			System.out.println(strcomm);

			try {
				out.println(strcomm);
			} catch (Exception ex) {}
		}
	}
}
