/***************************************************************************
 *   Copyright (C) 2002 by Rodolfo Conde                                   *
 *   aragorn@ciencias.unam.mx                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

 import java.util.Iterator;
 
/** The robot of the actual player (the program :) ) */
public class Robot extends SimpleRobot {
	/** The carrying capacity of this robot */
	private int c_capacity;
	
	/** the money */
	private int money;
	
	/** make a new robot with all the given parameters
	 *
	 * @param id the identifier of the robot
	 * @param x initial x position
	 * @param y initial y position
	 * @param ccap maximum carrying capacity
	 * @param mny amount of money
	 *
	 */
	 public Robot(int id, int x, int y, int ccap, int mny)
	 {
	 	super(id, x, y);
		c_capacity = ccap;
		money = mny;
	 }
	 
	 /** makes a robot with the given id, carrying capacity
	  * and some money.
		* @param id the identifier
		* @param ccap the carrying capacity
		* @param mny amount of money
		*
		*/
		public Robot(int id, int ccap, int mny)
		{
			this(id, 0, 0, ccap, mny);
		}
		
		/** Default constructor */
		public Robot()
		{
			this(0, 0, 0, 0, 0);
		}
		
	 /**
	  * sets the carrying capacity of the robot
		* 
		*@param ccap carrying capacity
		*
		*/
		public void setCarryingCapacity(int ccap)
		{
			c_capacity = ccap;
		}
		
	 /**
	  * Returns the carrying capacity of the robot
		* 
		*@return the carrying capacity
		*
		*/
		public int carryingCapacity()
		{
			return c_capacity;
		}
		
	 /**
	  * sets the money of the robot
		* 
		*@param mny the amount of money
		*
		*/
		public void setMoney(int mny)
		{
			money = mny;
		}
		
 /**
	  * returns the money of the robot
		* 
		*@return the amount of money of the robot
		*
		*/
		public int money()
		{
			return money;
		}
		
		/** executes the given command
		 * @param cmd the command to execute
		 *
		 */
		 public void execute(Command cmd)
		 {
		 	int which = cmd.id();
			
				if (which == Command.MOVE) {
					x_pos = cmd.x();
					y_pos = cmd.y();
				} else {
					Iterator it = cmd.arguments();
					int pw = 0;
					
					if (which == Command.PICK) {
						while (it.hasNext()) {
							pw = ((Package) it.next()).weight();
							
							if (c_capacity - pw >= 0) {
								c_capacity -= pw;
							}
							else {
								it.remove();  // hope its implemented :|
							}
						}
					} else {
						while (it.hasNext()) {
							pw = ((Package) it.next()).weight();
							c_capacity += pw;
					}
				}
		 }
		 
		 money -= cmd.bid();
	}
}
